#!/usr/bin/env ruby

require "#{File.dirname( File.expand_path(__FILE__))}/common"
include Common

class MainWin
	attr_reader :layout

	def initialize(window, skin, amarok, log)
		@window, @amarok, @log = window, amarok, log
		@layout = Gtk::Fixed.new

		if(set_skin(skin))
			@log.info('Skin ' + skin + ' loaded in mainwin.')
		else
			@log.fatal('Default skin ' + skin + ' not found, aborting.')
			quit
		end
	end

	def set_skin(skin)
		imagedata = {
			normalize_filename("#{File.dirname( File.expand_path(__FILE__))}/skins/#{skin}/Main.bmp") => {
		#		:name => [srcx, srcy, destx, desty,
		#			width, height]
				:main => [0, 0, 0, 0, 275, 116]
			},
			normalize_filename("#{File.dirname( File.expand_path(__FILE__))}/skins/#{skin}/Titlebar.bmp") => {
				:titlebar => [27, 0, 0, 0, 275, 14]
			}
		}
		buttondata = {
			normalize_filename("#{File.dirname( File.expand_path(__FILE__))}/skins/#{skin}/cButtons.bmp") => {
		#		:name => [srcx, srcy, destx, desty,
		#			offx, offy, width, height, proc*]
				:prev => [0, 0, 16, 88, 0, 18, 23, 18,
					proc {@amarok.prev}],
				:play => [23, 0, 39, 88, 0, 18, 23, 18,
					proc {@amarok.play}],
				:pause => [46, 0, 62, 88, 0, 18, 23, 18,
					proc {@amarok.toggle_pause}],
				:stop => [69, 0, 85, 88, 0, 18, 23, 18,
					proc {@amarok.stop}],
				:next => [92, 0, 108, 88, 0, 18, 22, 18,
        				proc {@amarok.next}],
				:open => [114, 0, 136, 89, 0, 16, 22, 16]
			},
			normalize_filename("#{File.dirname( File.expand_path(__FILE__))}/skins/#{skin}/Titlebar.bmp") => {
				:menu => [0, 0, 6, 3, 0, 9, 9, 9],
				:minimize => [9, 0, 244, 3, 0, 9, 9, 9,
					proc {@window.iconify}],
				:shade => [0, 18, 254, 3, 9, 0, 9, 9],
				:exit => [18, 0, 264, 3, 0, 9, 9, 9,
					proc {@window.destroy}]
			}
		}
		focusdata = {
			normalize_filename("#{File.dirname( File.expand_path(__FILE__))}/skins/#{skin}/Titlebar.bmp") => {
		#		:name => [srcx, srcy, destx, desty, offx,
		#			offy, width, height]
				:titlebar => [27, 0, 0, 0, 0, 15, 275, 14]
			}
		}
		hsliderdata = {
			normalize_filename("#{File.dirname( File.expand_path(__FILE__))}/skins/#{skin}/posBar.bmp") => {
		#		:groove => [srcx, srcy, destx, desty, width, height]
				:groove => [0, 0, 16, 72, 248, 10],
		#		:slider => [srcx, srcy, destx, desty, offx,
		#			offy, width, height]
				:slider => [248, 0, 32, 72, 30, 0, 29, 10]
			}
		}

		[imagedata, buttondata, focusdata, hsliderdata].each do |hash|
			if(hash.has_key?(nil))
				return false
			end
		end
		images(imagedata, @layout)
		buttons(buttondata, @layout)
		focusable(focusdata, @layout, @window)
		hslider(hsliderdata, @layout)

		return true
	end
end



