#!/usr/bin/env ruby

require 'gui/common'
include Common

class MainWin
	attr_reader :layout

	def initialize(window, skin, xmms, log)
		@window, @xmms, @log = window, xmms, log
		@layout = Gtk::Fixed.new

		if(set_skin(skin))
			@log.info('Skin ' + skin + ' loaded in mainwin.')
		else
			@log.fatal('Default skin ' + skin + ' not found, aborting.')
			quit
		end
	end

	def set_skin(skin)
		imagedata = {
			normalize_filename("skins/#{skin}/Main.bmp") => {
		#		:name => [srcx, srcy, destx, desty,
		#			width, height]
				:main => [0, 0, 0, 0, 275, 116]
			},
			normalize_filename("skins/#{skin}/Titlebar.bmp") => {
				:titlebar => [27, 0, 0, 0, 275, 14]
			}
		}
		buttondata = {
			normalize_filename("skins/#{skin}/cButtons.bmp") => {
		#		:name => [srcx, srcy, destx, desty,
		#			offx, offy, width, height, proc*]
				:prev => [0, 0, 16, 88, 0, 18, 23, 18,
					method(:wrap_prev)],
				:play => [23, 0, 39, 88, 0, 18, 23, 18,
					proc {@xmms.play}],
				:pause => [46, 0, 62, 88, 0, 18, 23, 18,
					proc {@xmms.toggle_pause}],
				:stop => [69, 0, 85, 88, 0, 18, 23, 18,
					proc {@xmms.stop}],
				:next => [92, 0, 108, 88, 0, 18, 22, 18,
					method(:wrap_next)],
				:open => [114, 0, 136, 89, 0, 16, 22, 16]
			},
			normalize_filename("skins/#{skin}/Titlebar.bmp") => {
				:menu => [0, 0, 6, 3, 0, 9, 9, 9],
				:minimize => [9, 0, 244, 3, 0, 9, 9, 9,
					proc {@window.iconify}],
				:shade => [0, 18, 254, 3, 9, 0, 9, 9],
				:exit => [18, 0, 264, 3, 0, 9, 9, 9,
					proc {@window.destroy}]
			}
		}
		focusdata = {
			normalize_filename("skins/#{skin}/Titlebar.bmp") => {
		#		:name => [srcx, srcy, destx, desty, offx,
		#			offy, width, height]
				:titlebar => [27, 0, 0, 0, 0, 15, 275, 14]
			}
		}
		hsliderdata = {
			normalize_filename("skins/#{skin}/posBar.bmp") => {
		#		:groove => [srcx, srcy, destx, desty, width, height]
				:groove => [0, 0, 16, 72, 248, 10],
		#		:slider => [srcx, srcy, destx, desty, offx,
		#			offy, width, height]
				:slider => [248, 0, 32, 72, 30, 0, 29, 10]
			}
		}

		[imagedata, buttondata, focusdata, hsliderdata].each do |hash|
			if(hash.has_key?(nil))
				return false
			end
		end
		images(imagedata, @layout)
		buttons(buttondata, @layout)
		focusable(focusdata, @layout, @window)
		hslider(hsliderdata, @layout)

		return true
	end
end

class Equalizer
	attr_reader :layout

	def initialize(window, skin, xmms, log)
		@window, @xmms, @log = window, xmms, log
		@layout = Gtk::Fixed.new

		if(set_skin(skin))
			@log.info('Skin ' + skin + ' loaded in equalizer.')
		else
			@log.fatal('Default skin ' + skin + ' not found, aborting.')
			quit
		end
	end

	def set_skin(skin)
		imagedata = {
			normalize_filename("skins/#{skin}/Eqmain.bmp") => {
		#		:name => [srcx, srcy, destx, desty,
		#			width, height]
				:titlebar => [0, 0, 0, 0, 275, 14],
				:main => [0, 14, 0, 14, 275, 102]
			}
		}
		buttondata = {}
		focusdata = {
			normalize_filename("skins/#{skin}/Eqmain.bmp") => {
		#		:name => [srcx, srcy, destx, desty, offx,
		#			offy, width, height]
				:titlebar => [0, 134, 0, 0, 0, 15, 275, 14]
			}
		}

		[imagedata, buttondata, focusdata].each do |hash|
			if(hash.has_key?(nil))
				return false
			end
		end
		images(imagedata, @layout)
		buttons(buttondata, @layout)
		focusable(focusdata, @layout, @window)

		return true
	end
end

require 'format'
require 'gui/skinfo'

class Playlist
	attr_reader :layout

	def initialize(window, skin, format, xmms, log)
		@window, @format, @xmms, @log = window, format, xmms, log
		@layout = Gtk::Fixed.new
		@formatter = Format.new(@format)
		@skinfo = Skinfo.new(skin, @log)
		@fallback = nil

		if(set_skin(skin))
			@log.info('Skin ' + skin + ' loaded in playlist.')
		else
			@log.fatal('Default skin ' + skin + ' not found, aborting.')
			quit
		end
		disp_playlist
	end

	def set_skin(skin)
		imagedata = {
			normalize_filename("skins/#{skin}/Pledit.bmp") => {
		#		:name => [srcx, srcy, destx, desty,
		#			width, height]
				:lfill1 => [0, 42, 0, 20, 12, 29],
				:lfill2 => [0, 42, 0, 49, 12, 29],
				:lfill3 => [0, 42, 0, 78, 12, 29],
				:lfill4 => [0, 42, 0, 107, 12, 29],
				:lfill5 => [0, 42, 0, 136, 12, 29],
				:lfill6 => [0, 42, 0, 165, 12, 29],
				:rlfill1 => [31, 42, 255, 20, 5, 29],
				:rlfill2 => [31, 42, 255, 49, 5, 29],
				:rlfill3 => [31, 42, 255, 78, 5, 29],
				:rlfill4 => [31, 42, 255, 107, 5, 29],
				:rlfill5 => [31, 42, 255, 136, 5, 29],
				:rlfill6 => [31, 42, 255, 165, 5, 29],
				:rfill1 => [36, 42, 260, 20, 8, 29],
				:rfill2 => [36, 42, 260, 49, 8, 29],
				:rfill3 => [36, 42, 260, 78, 8, 29],
				:rfill4 => [36, 42, 260, 107, 8, 29],
				:rfill5 => [36, 42, 260, 136, 8, 29],
				:rfill6 => [36, 42, 260, 165, 8, 29],
				:rrfill1 => [44, 42, 268, 20, 7, 29],
				:rrfill2 => [44, 42, 268, 49, 7, 29],
				:rrfill3 => [44, 42, 268, 78, 7, 29],
				:rrfill4 => [44, 42, 268, 107, 7, 29],
				:rrfill5 => [44, 42, 268, 136, 7, 29],
				:rrfill6 => [44, 42, 268, 165, 7, 29],
				:controls => [0, 72, 0, 194, 125, 38],
				:minicont => [126, 72, 125, 194, 150, 38]
			}
		}
		buttondata = {
			normalize_filename("skins/#{skin}/Pledit.bmp") => {
		#		:name => [srcx, srcy, destx, desty,
		#			offx, offy, width, height, proc*]
				:prev => [132, 94, 131, 216, 0, 0, 9, 8,
					method(:wrap_prev)],
				:play => [140, 94, 139, 216, 0, 0, 8, 8,
					proc {@xmms.play}],
				:pause => [149, 94, 148, 216, 0, 0, 9, 8,
					proc {@xmms.toggle_pause}],
				:stop => [159, 94, 158, 216, 0, 0, 9, 8,
					proc {@xmms.stop}],
				:next => [167, 94, 166, 216, 0, 0, 9, 8,
					method(:wrap_next)],
				:open => [177, 94, 176, 216, 0, 0, 9, 8]
			}
		}
		focusdata = {
		#	'filename' => {
			normalize_filename("skins/#{skin}/Pledit.bmp") => {
		#		:name => [srcx, srcy, destx, desty,
		#			offx, offy, width, height]
				:titlebar => [26, 0, 25, 0, 0, 21, 100, 20],
				:tfill1 => [127, 0, 125, 0, 0, 21, 25, 20],
				:tfill2 => [127, 0, 150, 0, 0, 21, 25, 20],
				:tfill3 => [127, 0, 175, 0, 0, 21, 25, 20],
				:tfill4 => [127, 0, 200, 0, 0, 21, 25, 20],
				:tfill5 => [127, 0, 225, 0, 0, 21, 25, 20],
				:ulcorner => [0, 0, 0, 0, 0, 21, 25, 20],
				:urcorner => [153, 0, 250, 0, 0, 21, 25, 20]
			}
		}

		[imagedata, buttondata, focusdata].each do |hash|
			if(hash.has_key?(nil))
				return false
			end
		end
		images(imagedata, @layout)
		buttons(buttondata, @layout)
		focusable(focusdata, @layout, @window)

		return true
	end

	def set_fallback_encoding(encoding)
		@fallback = encoding
	end

	# Gtk+ has again made me want to vomit and die. There are about 5 or 6
	# different objects, widgets, and just classes in general needed to make
	# it understand how to draw a simple playlist. Ask the Gnome devs why they
	# thought it was a good idea to use CellRenderers, CellRendererTexts,
	# TreeViews, TreeViewColumns, TreeIters, ListStores, and Gdk::Colors to
	# draw and color a simple table. And you wonder why it takes me so long
	# before I release new GUI code for Rxmms2! :)
	def disp_playlist
		@view = Gtk::TreeView.new
		@store = Gtk::ListStore.new(String, String)
		# You need one of these...
		renderer = Gtk::CellRendererText.new
		# ...to define one of these. (GTK!!!)
		title = Gtk::TreeViewColumn.new('Title', renderer, :text => 0)
		renderer = Gtk::CellRendererText.new
		renderer.set_xalign(1) # Right-align the playtimes.
		length = Gtk::TreeViewColumn.new('Length', renderer, :text => 1)

		# I still have to figure out how to allow the playtime column to grow
		# dynamically and keep the title column from pushing it off of the
		# TreeView. Gtk+ allows you to lock the first column into place, but no
		# other. (GTK!!!)
		title.set_fixed_width(243 - 30)
		length.set_fixed_width(30)
		# Here's another Gtk-ism. The row height stays the same size, even if
		# you decrease the font size. This was an attempt to fix the problem,
		# but for some reason, setting padding sizes to nothing, actually
		# doesn't do anything at all. (GTK!!!)
		[title, length].each do |col|
			col.set_sizing(Gtk::TreeViewColumn::FIXED)
			col.cell_renderers[0].set_xpad(0)
			col.cell_renderers[0].set_ypad(0)
			col.cell_renderers[0].set_size_points(10)
			col.cell_renderers[0].set_fixed_height_from_font(1)
			col.set_spacing(0)
			@view.append_column(col)
		end

		plinfo = @skinfo.plinfo # Get playlist font information from Skinfo.

		# Please don't actually reorder anything yet, it won't be reflected in
		# XMMS2's playlist. :)
		@view.set_reorderable(true)
		# This doesn't work, either, you can only select one item. (GTK!!!)
		@view.selection.set_mode(Gtk::SELECTION_MULTIPLE)
		@view.set_size_request(243, 174) # These actually do what they're
		@view.set_headers_visible(false) # supposed to.
		# Here's where we change the background and text colors we got from
		# Skinfo. I still have to figure this out when it's highlighted.
		@view.modify_base(@view.state, Gdk::Color.parse(plinfo['NormalBG']))
		@view.modify_text(@view.state, Gdk::Color.parse(plinfo['Normal']))
		@view.signal_connect('row-activated') do |view, path, column|
				@xmms.jump(path.indices[0])
				@xmms.play unless(@xmms.playing?)
		end
		# Set up callbacks to keep the playlist up-to-date.
		@xmms.register_callback(:connected, &method(:on_connect))
		@xmms.register_callback(:playlist, &method(:update_playlist))
		@xmms.register_callback(:medialib_entry, &method(:update_mediainfo))
		# And put the initial playlist in place.
		init_playlist
		# Finally, put this ugly thing into our playlist window.
		@layout.put(@view, 12, 20)
	end

	private

	def init_playlist
		@store.clear
		@xmms.playlist.each do |info|
			update_playlist({:type => Xmms2::ADD}, info)
		end
	end

	def update_playlist(type, info)
		case(type[:type])
			when(Xmms2::ADD)
				disp_str = format_data(info)
				iter = @store.append
				iter[0] = (iter.path.to_str.to_i + 1).to_s + '. ' + disp_str
				iter[1] = format_time(info[:server][:duration] || 0)
			when(Xmms2::INS)
				disp_str = format_data(info)
				iter = @store.insert(type[:pos])
				iter[0] = (type[:pos] + 1).to_s + '. ' + disp_str
				iter[1] = format_time(info[:server][:duration] || 0)

				renumber_list(type[:pos])
			when(Xmms2::MOV)
				if(type[:old] > type[:new])
					new = @store.insert(type[:new])
					type[:old] += 1
				else
					new = @store.insert(type[:new] + 1)
				end
				old = @store.get_iter(type[:old].to_s)
				new[0] = old[0]
				new[1] = old[1]

				@store.remove(old)
				renumber_list([type[:old], type[:new]].min)
			when(Xmms2::REM)
				@store.remove(@store.get_iter(type[:pos].to_s))
				if(type[:pos] != @xmms.length)
					renumber_list(type[:pos])
				end
			when(Xmms2::CLR)
				@store.clear
			when(Xmms2::SRT)
				init_playlist
			when(Xmms2::SHF)
				init_playlist
		end
		@view.set_model(@store)
	end

	def update_mediainfo(info)
		@xmms.get_pos_from_id(info[:server][:id]).each do |pos|
			iter = @store.get_iter(pos.to_s)
			display_string = format_data(info, true)
			next unless(iter)

			iter[0] = (pos + 1).to_s + '. ' + display_string
			iter[1] = format_time(info[:server][:duration])
		end
		@view.set_model(@store)
	end

	def format_data(info, ignore = false)
		@log.debug('Ignoring display_string: ' + ignore.to_s)
		key = "client/#{@xmms.client}".to_sym

		if(!ignore && info.has_key?(key) &&
			info[key].has_key?(:display_string) && info[key][:display_format] ==
			@format)
			disp_str = info[key][:display_string]
			@log.debug('Display string found: ' + disp_str)
		elsif(!info[:server].has_key?(:url_decoded))
			@log.debug('No decoded URL found, file is probably unresolved.')
			disp_str = ''
		else
			# Heuristics to validate and convert filename to UTF-8.
			unless(GLib.utf8_validate(info[:server][:url_decoded]))
				decoded = nil
				[@fallback, GLib.charset].compact.each do |charset|
					decoded = try_convert(info[:server][:url_decoded], charset)
					if(decoded)
						@log.debug("URL converted from #{charset} to UTF-8")
						info[:server][:url_decoded] = decoded
						break
					end
				end
				unless(decoded)
					@log.info('Decoded URL could not be converted to UTF-8: ' +
						info[:server][:url_decoded])
					info[:server][:url_decoded] = 'Invalid UTF-8'
				end
			end
			disp_str = @formatter.prettify(info[:server])
			unless(disp_str)
				@log.fatal("Display string didn't evaluate, check your config!")
				exit
			end
			@log.debug('New display string created: ' + disp_str)

			{
				'display_format' => @format,
				'display_string'=> disp_str
			}.each_pair do |key, val|
				@xmms.medialib_add_key(info[:server][:id], key, val)
			end
		end

		return disp_str
	end

	def format_time(time)
		return '' unless(time)
		# The +5 rounds up half seconds after the %600 (more accurate).
		return sprintf('%d:%02d', time / 1000 / 60, (time / 100 + 5) % 600 / 10)
	end

	def renumber_list(pos)
		return if(pos >= @xmms.playlist.length - 1)
		@log.debug('Reordering GUI playlist from pos: ' + pos.to_s)
		iter = @store.get_iter(pos.to_s)
		iter = @store.append(nil) unless(iter)

		begin
			pos += 1
			str = ''
			i = 0

			while(i < iter[0].length)
				i += 1
				next unless(iter[0][i].chr == '.')
				str = iter[0][i...iter[0].length]

				break
			end
			iter[0] = pos.to_s + str
		end while(iter.next!)
	end

	def try_convert(str, charset)
		begin
			@log.debug('Attempting conversion from ' + charset + ' to UTF-8.')
			return GLib.convert(str, 'UTF-8', charset)
		rescue GLib::ConvertError
			@log.debug('Conversion from ' + charset + ' to UTF-8 failed.')
			return nil
		end
	end

	def on_connect
		update_playlist({:type => Xmms2::CLR}, nil)
	end
end
