#!/usr/bin/env ruby

class Log
	def initialize(logpath, overwrite=true, verbosity = :info)
		@verbosity = verbosity
		@verbosities = {
			:fatal => 0,
			:error => 1,
			:warn => 2,
			:info => 3,
			:debug => 4
		}

		if(open(logpath, overwrite))
			@file.puts timestamp + ' ***INITIALIZED***'
			return true
		else
			return false
		end
	end

	def fatal(msg)
		puts(:fatal, msg)
	end

	def error(msg)
		puts(:error, msg)
	end

	def warn(msg)
		puts(:warn, msg)
	end

	def info(msg)
		puts(:info, msg)
	end

	def debug(msg)
		puts(:debug, msg)
	end

	def close
		unless(@file.closed?)
			@file.puts timestamp + ' ***CLOSED***'
			@file.close
		end
	end

	private

	def open(filename, overwrite)
		return false unless(filename)
		if(overwrite)
			mode = File::WRONLY | File::CREAT | File::TRUNC
		elsif(!overwrite)
			mode = File::WRONLY | File::APPEND
			File.new(filename, File::CREAT) unless(File.exists?(filename))
		end

		@file = File.new(filename, mode)
		@file.sync = true

		return true
	end

	def puts(level, msg)
		if(@verbosities[@verbosity] >= @verbosities[level] && !@file.closed?)
			@file.puts timestamp + level.to_s.upcase + ': ' + msg
		end
	end

	def timestamp
		return Time.now.strftime('%H:%M:%S ')
	end
end
