#!/usr/bin/env ruby

# This class is used to log all events that the Xmms2 class handles. This keeps
# the Xmms2 class nice and clean and general-purpose.

class Xmms2Log
	def initialize(xmms, log)
		@xmms, @log = xmms, log
		@callbacks = []

		broadcasts
	end

	private

	def broadcasts
		{
			:current_id => 'Current ID: ',
			:current_pos => 'Current pos: '
		}.each_pair do |key, val|
			@callbacks.push(@xmms.register_callback(key) do |ret|
				@log.debug(val + ret.to_s)
			end)
		end

		configval_broadcast
		medialib_broadcast
		playlist_broadcast
		status_broadcast
	end

	def configval_broadcast
		@callbacks.push(@xmms.register_callback(:configval) do |config|
			config.each_pair do |key, val|
				@log.debug("Configval changed: #{key} => #{val}")
			end
		end)
	end

	def medialib_broadcast
		@callbacks.push(@xmms.register_callback(:medialib_entry) do |info|
			@log.debug("Mediainfo updated for id: #{info[:server][:id]}")
		end)
	end

	def playlist_broadcast
		@callbacks.push(@xmms.register_callback(:playlist) do |type, info|
			case(type[:type])
				when(Xmms2::ADD)
					@log.debug("Playlist added ID: #{info[:server][:id]}; " +
						"length: #{@xmms.length}")
				when(Xmms2::INS)
					@log.debug("Playlist inserted ID: #{info[:server][:id]}; " +
						"pos: #{type[:pos]}")
				when(Xmms2::MOV)
					@log.debug("Playlist moved pos: #{type[:old]} to: " + 
						"#{type[:new]}")
				when(Xmms2::REM)
					if(info[:server])
						@log.debug("Playlist removed ID: #{info[:server][:id]}; " +
							"pos: #{type[:pos]}")
					else
						@log.debug("Playlist removed pos: #{type[:pos]}")
					end
				when(Xmms2::CLR)
					@log.debug('Playlist cleared.')
				when(Xmms2::SRT)
					@log.debug('Playlist sorted.')
				when(Xmms2::SHF)
					@log.debug('Playlist shuffled.')
				when(Xmms2::UNKNOWN)
					@log.error('Unknown (possibly unresolved) playlist update.')
			end
		end)
	end

	def status_broadcast
		@callbacks.push(@xmms.register_callback(:playback_status) do |status|
			case(status)
				when(Xmms2::PLAYING)
					@log.debug('Playback status: playing.')
				when(Xmms2::PAUSED)
					@log.debug('Playback status: paused.')
				when(Xmms2::STOPPED)
					@log.debug('Playback status: stopped.')
				when(Xmms2::UNKNOWN)
					@log.debug('Playback status: unknown.')
			end
		end)
	end
end
